#!/usr/bin/ruby

require 'kconv'
require 'csv'
require 'standard/swrba_command'          # Command Class
require 'standard/swrba_logger'           # Logger Class
require 'standard/swrba_def'              # Common Definition Class
require 'standard/swrba_input'            # Input Information File Analysis Class
require 'standard/swrba_vserver_msgdef'   # vserver message 
require 'standard/swrba_vserver_vmhost'   # vserver vmhost
require 'standard/swrba_multicall'        # Method multi call Class

#=vmhost Server Operation Parts
#Version::   V14.1.0-V14.1.0A
#Parts::     0002
#Copyright:: Copyright (C) FUJITSU LIMITED 2010-2011
#
#=== Function Contents
#* Operate vmhost Server

# Const value


# Logger Class Instance
logger = Swrba_Logger.new("swrba_vmhost_vm_shutdown", Swrba_Def::SWRBA_FILE_LOG)

logger.warn(":[begin]")

# Input Information File Analysis Class
input = Swrba_Input.new

# Initialize
option = []

# Get Option Information
while data = ARGV.shift
  option.push(data)
end

# Check Parameter Number
if option.size != 2
  logger.warn("parameter check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check Option Form
if option[0] != "-input"
  logger.warn("option check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check File Exist
if File.exist?(option[1]) == false
  logger.warn("file exist error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Input File Name
logger.warn("Input File Name = " + option[1])

# Get Input Information
begin
  hostname     = input.input_analysis(option[1], "hostname")
  ipaddress    = input.input_analysis(option[1], "ipaddress")
  vmhost     = input.input_analysis(option[1], "vmhost")
  execusername = input.input_analysis(option[1], "execusername")
  execpassword = input.input_analysis(option[1], "execpassword")
  notarget = input.input_analysis(option[1], "notarget")
  operationmode   = input.input_analysis(option[1], "operationmode")
rescue
  logger.warn("input analysis error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_RETRY
end

# Priority to IP Address
if ipaddress != nil && ipaddress != ""
  hostname = ipaddress
end

# Check Required
if hostname == nil
  logger.warn("hostname None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

if vmhost == nil
  logger.warn("vmhost None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

if execusername == nil || execusername == ""
  logger.warn("execusername None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

if execpassword == nil || execpassword == ""
  logger.warn("execpassword None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

begin
  if notarget != nil && notarget != ""
    notarget = notarget.toutf8
    notarget_arg = notarget.to_s.split(/,/)
    escape_num = 0

    charnum = 0

    notarget_size = notarget_arg.size
    i = 0
    while i < notarget_arg.size
      escape_num = 0
      notarget_arg[i].each_byte {|c|
        if c.chr == '\\'
          escape_num = escape_num + 1
        else
          escape_num = 0
        end
        
        charnum = charnum + 1

      }
      if escape_num % 2 == 1 && i + 1 < notarget_arg.size
        notarget_size = notarget_size - 1
      
      else
        if charnum == 0
          raise
        end
        charnum = 0
      end
      i = i + 1
    end
    
    if notarget_size < 1 || notarget_size > 100
      raise
    end
  else
    notarget = ""
  end
rescue
  logger.warn("notarget invalid")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

ary_vmhost = []
CSV.parse(vmhost){|tmpstr|
  ary_vmhost += tmpstr
}
if ary_vmhost.length > 100
  logger.warn("vmhost exceeded the limitation.")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

result = Swrba_Def::NORMAL_CODE
if ary_vmhost.length == 1
  # operation vmhost
  vmhost = ary_vmhost[0]
  vmhost = Swrba_Vserver_VMHost.new(ipaddress, vmhost, execusername, execpassword)
  begin
    vmhost.vmhost_vmshutdown(notarget)
  rescue => exc
    logger.warn("vmhost_vmshutdown error")
    logger.warn(exc)
    logger.warn(":[error]")
    STDERR.puts vmhost.err_msg.toutf8
    exit vmhost.return_code
  end

  result = Swrba_Def::NORMAL_CODE
  # Standard Error Output
  STDERR.puts "Success"
else
  # operation vmhost multi
  hostname = Array.new(ary_vmhost.length, hostname)
  execusername = Array.new(ary_vmhost.length, execusername)
  execpassword = Array.new(ary_vmhost.length, execpassword)
  notarget = Array.new(ary_vmhost.length, notarget)

  multicall = Swrba_multicall.new
  vmhost_obj = Swrba_Vserver_VMHost.new

  begin
    multicall.multicall(vmhost_obj, vmhost_obj.method(:vmhost_vmshutdown_multi), operationmode, "vmhost", ary_vmhost, hostname, ary_vmhost, execusername, execpassword, notarget)
  rescue => exc
    STDERR.puts exc.to_s.toutf8
    logger.warn(exc)
    logger.warn(":[error]")
    exit multicall.multi_return_code
  end

  STDOUT.print multicall.multi_stdout.toutf8
  STDERR.puts multicall.multi_stderr.toutf8
  result = multicall.multi_return_code
  logger.warn("return_code=>#{result}")
  logger.warn("stdout=>#{multicall.multi_stdout}")
  logger.warn("stderr=>#{multicall.multi_stderr}")
end

logger.warn(":[end]")

exit result	
